@@ -57,12 +57,22 @@ const float PHASE_SCALE   = 0.5f;
 const float COARSE_SCALE  = 12.0f;
 const float FINE_SCALE    = 1.0f;
 const float SWEEP_SCALE   = 0.5f;
+const float PITCH_SCALE   = 0.5f;
 
 const float LFO_FREQ_MIN  = 0.4f;
 const float LFO_FREQ_MAX  = 40.0f;
 
 
+// maximum helper
+
+inline float drumkv1_max ( float a, float b )
+{
+	return (a > b ? a : b);
+}
+
+
 // hyperbolic-tangent fast approximation
+
 inline float drumkv1_tanhf ( const float x )
 {
 	const float x2 = x * x;
@@ -82,8 +92,6 @@ inline float drumkv1_sigmoid_0 ( const float x, const float t0 )
 {
 	const float t1 = 1.0f - t0;
 #if 0
-	return (x < -1.0f ? -t1 : (x > +1.0f ? t1 : t1 * x * (1.5f - 0.5f * x * x)));
-#else
 	if (x > +t1)
 		return +t1 + t0 * drumkv1_tanhf(+(x - t1) / t0);
 	else
@@ -91,6 +99,8 @@ inline float drumkv1_sigmoid_0 ( const float x, const float t0 )
 		return -t1 - t0 * drumkv1_tanhf(-(x + t1) / t0);
 	else
 		return x;
+#else
+	return (x < -1.0f ? -t1 : (x > +1.0f ? t1 : t1 * x * (1.5f - 0.5f * x * x)));
 #endif
 }
 
@@ -108,6 +118,25 @@ inline float drumkv1_velocity ( const float x, const float p = 0.2f )
 }
 
 
+// pitchbend curve
+
+inline float drumkv1_pow2f ( const float x )
+{
+// simplest power-of-2 straight linearization
+// -- x argument valid in [-1, 1] interval
+//	return 1.0f + (x < 0.0f ? 0.5f : 1.0f) * x;
+	return ::powf(2.0f, x);
+}
+
+
+// convert note to frequency (hertz)
+
+inline float drumkv1_freq ( float note )
+{
+	return (440.0f / 32.0f) * ::powf(2.0f, (note - 9.0f) / 12.0f);
+}
+
+
 // envelope
 
 struct drumkv1_env
@@ -251,7 +280,7 @@ struct drumkv1_ctl
 	void reset()
 	{
 		pressure = 0.0f;
-		pitchbend = 0.0f;
+		pitchbend = 1.0f;
 		modwheel = 0.0f;
 		panning = 0.0f;
 		volume = 1.0f;
@@ -519,7 +548,7 @@ public:
 		m_t1 = m_b3; m_b3 = (m_b2 + m_t2) * m_p - m_b3 * m_f;
 
 		m_b4 = (m_b3 + m_t1) * m_p - m_b4 * m_f;
-	//	m_b4 = m_b4 - m_b4 * m_b4 * m_b4 * 0.166667f;	-- clipping
+		m_b4 = m_b4 - m_b4 * m_b4 * m_b4 * 0.166667f;	// clipping
 
 		m_b0 = input;
 
@@ -556,9 +585,9 @@ public:
 
 protected:
 
-	virtual float evaluate(uint16_t i)
+	float evaluate(uint16_t i)
 	{
-		drumkv1_ramp3::evaluate(i);
+		drumkv1_ramp3::update();
 
 		const float wpan = 0.25f * M_PI
 			* (1.0f + m_param1_v)
@@ -571,12 +600,23 @@ protected:
 
 
 
-// convert note to frequency (hertz)
+// pressure smoother (3 parameters)
 
-inline float note_freq ( float note )
+class drumkv1_pre : public drumkv1_ramp3
 {
-	return (440.0f / 32.0f) * ::powf(2.0f, (note - 9.0f) / 12.0f);
-}
+public:
+
+	drumkv1_pre() : drumkv1_ramp3() {}
+
+protected:
+
+	float evaluate(uint16_t)
+	{
+		drumkv1_ramp3::update();
+
+		return m_param1_v * drumkv1_max(m_param2_v, m_param3_v);
+	}
+};
 
 
 // synth element
@@ -612,8 +652,8 @@ drumkv1_elem::drumkv1_elem ( uint32_t iSampleRate, int key )
 	: element(this)
 {
 	// element parameter value set
-	for (int i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i)
-		for (int j = 0; j < 3; ++j)	params[j][i] = 0.0f;
+	for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i)
+		for (int j = 0; j < 3; ++j) params[j][i] = 0.0f;
 
 	// element key (sample note)
 	gen1.sample0 = float(key);
@@ -665,17 +705,20 @@ struct drumkv1_voice : public drumkv1_list<drumkv1_voice>
 	int note;									// voice note
 	int group;									// voice group
 	float vel;									// velocity to vol
+	float pre;									// key pressure/aftertouch
 
 	float gen1_freq;							// frequency and phase
 
 	float lfo1_sample;
 
 	drumkv1_filter1 dcf11, dcf12;				// filters
-	drumkv1_filter1 dcf13, dcf14;
+	drumkv1_filter2 dcf13, dcf14;
 
 	drumkv1_env::State dca1_env;				// envelope states
 	drumkv1_env::State dcf1_env;
 	drumkv1_env::State lfo1_env;
+
+	drumkv1_pre dca1_pre;
 };
 
 
@@ -754,7 +797,9 @@ private:
 	uint32_t m_iSampleRate;
 
 	drumkv1_ctl m_ctl;
+
 	drumkv1_def m_def;
+
 	drumkv1_cho m_cho;
 	drumkv1_fla m_fla;
 	drumkv1_pha m_pha;
@@ -775,8 +820,6 @@ private:
 
 	drumkv1_list<drumkv1_elem>  m_elem_list;
 
-	drumkv1_ramp2 m_pre;
-
 	drumkv1_fx_chorus   m_chorus;
 	drumkv1_fx_flanger *m_flanger;
 	drumkv1_fx_phaser  *m_phaser;
@@ -967,6 +1010,7 @@ void drumkv1_impl::setCurrentElement ( int key )
 					element->setParamPort(index, &(elem->params[1][i]));
 				}
 			}
+			resetElement(elem);
 		}
 		// swap new element parameter port values
 		elem = m_elems[key];
@@ -982,9 +1026,9 @@ void drumkv1_impl::setCurrentElement ( int key )
 					element->setParamPort(index, pfParam);
 				}
 			}
+			resetElement(elem);
 		}
 		// set new current element
-		resetElement(elem);
 		m_elem = elem;
 	}
 	else m_elem = 0;
@@ -1173,20 +1217,22 @@ void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			const float vel = float(value) / 127.0f;
 			// quadratic velocity law
 			pv->vel  = drumkv1_velocity(vel * vel, *m_def.velocity);
+			// pressure/aftertouch
+			pv->pre = 0.0f;
+			pv->dca1_pre.reset(m_def.pressure, &m_ctl.pressure, &pv->pre);
 			// generate
 			pv->gen1.start();
 			// frequencies
 			const float freq1 = float(key)
 				+ *elem->gen1.coarse * COARSE_SCALE
 				+ *elem->gen1.fine * FINE_SCALE;
-			pv->gen1_freq = note_freq(freq1);
+			pv->gen1_freq = drumkv1_freq(freq1);
 			// filters
-			const drumkv1_filter1::Type type1
-				= drumkv1_filter1::Type(int(*elem->dcf1.type));
-			pv->dcf11.reset(type1);
-			pv->dcf12.reset(type1);
-			pv->dcf13.reset(type1);
-			pv->dcf14.reset(type1);
+			const int type1 = int(*elem->dcf1.type);
+			pv->dcf11.reset(drumkv1_filter1::Type(type1));
+			pv->dcf12.reset(drumkv1_filter1::Type(type1));
+			pv->dcf13.reset(drumkv1_filter2::Type(type1));
+			pv->dcf14.reset(drumkv1_filter2::Type(type1));
 			// envelopes
 			elem->dcf1.env.start(&pv->dcf1_env);
 			elem->lfo1.env.start(&pv->lfo1_env);
@@ -1226,12 +1272,18 @@ void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			}
 		}
 	}
+	// key pressure/poly.aftertouch
+	else if (status == 0xa0) {
+		drumkv1_voice *pv = m_notes[key];
+		if (pv && pv->note >= 0)
+			pv->pre = *m_def.pressure * float(value) / 127.0f;
+	}
 	// control change
 	else if (status == 0xb0) {
 		switch (key) {
 		case 0x01:
 			// modulation wheel (cc#1)
-			m_ctl.modwheel = float(value) / 127.0f;
+			m_ctl.modwheel = *m_def.modwheel * float(value) / 127.0f;
 			break;
 		case 0x07:
 			// channel volume (cc#7)
@@ -1257,7 +1309,8 @@ void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
 	}
 	// pitch bend
 	else if (status == 0xe0) {
-		m_ctl.pitchbend = float(key + (value << 7) - 0x2000) / 8192.0f;
+		const float pitchbend = float(key + (value << 7) - 0x2000) / 8192.0f;
+		m_ctl.pitchbend = drumkv1_pow2f(*m_def.pitchbend * pitchbend);
 	}
 }
 
@@ -1315,13 +1368,11 @@ void drumkv1_impl::allNotesOff (void)
 
 void drumkv1_impl::resetElement ( drumkv1_elem *elem )
 {
-	if (elem) {
-		elem->vol1.reset(elem->out1.volume, elem->dca1.volume,
-			&m_ctl.volume, &elem->aux1.volume);
-		elem->pan1.reset(elem->out1.panning,
-			&m_ctl.panning, &elem->aux1.panning);
-		elem->wid1.reset(elem->out1.width);
-	}
+	elem->vol1.reset(elem->out1.volume, elem->dca1.volume,
+		&m_ctl.volume, &elem->aux1.volume);
+	elem->pan1.reset(elem->out1.panning,
+		&m_ctl.panning, &elem->aux1.panning);
+	elem->wid1.reset(elem->out1.width);
 }
 
 
@@ -1354,9 +1405,6 @@ void drumkv1_impl::reset (void)
 		elem = elem->next();
 	}
 
-	// pressure state
-	m_pre.reset(m_def.pressure, &m_ctl.pressure);
-
 	// flangers
 	if (m_flanger == 0)
 		m_flanger = new drumkv1_fx_flanger [m_iChannels];
@@ -1405,14 +1453,9 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 			elem->lfo1_wave.reset(
 				drumkv1_wave::Shape(*elem->lfo1.shape), *elem->lfo1.width);
 		}
-		elem->wid1.process(nframes);
-		elem->pan1.process(nframes);
-		elem->vol1.process(nframes);
 		elem = elem->next();
 	}
 
-	m_pre.process(nframes);
-
 	// per voice
 
 	drumkv1_voice *pv = m_play_list.next();
@@ -1424,11 +1467,9 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		// controls
 		drumkv1_elem *elem = pv->elem;
 
-		const float lfo1_rate  = *elem->lfo1.rate * *elem->lfo1.rate;
-		const float lfo1_freq  = LFO_FREQ_MIN + lfo1_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
-		const float lfo1_pitch = *elem->lfo1.pitch * *elem->lfo1.pitch;
-		const float modwheel1  = *m_def.modwheel * (lfo1_pitch + m_ctl.modwheel);
-		const float pitchbend1 = (1.0f + *m_def.pitchbend * m_ctl.pitchbend);
+		const float lfo1_rate = *elem->lfo1.rate * *elem->lfo1.rate;
+		const float lfo1_freq = LFO_FREQ_MIN + lfo1_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
+		const float modwheel1 = m_ctl.modwheel + PITCH_SCALE * *elem->lfo1.pitch;
 
 		// channel indexes
 
@@ -1460,14 +1501,15 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				// velocities
 
 				const float vel1
-					= (pv->vel + (1.0f - pv->vel) * m_pre.value(j));
+					= (pv->vel + (1.0f - pv->vel) * pv->dca1_pre.value(j));
 
 				// generators
 
 				const float lfo1_env = pv->lfo1_env.tick();
 				const float lfo1 = pv->lfo1_sample * lfo1_env;
 
-				pv->gen1.next(pv->gen1_freq * (pitchbend1 + modwheel1 * lfo1));
+				pv->gen1.next(pv->gen1_freq
+					* (m_ctl.pitchbend + modwheel1 * lfo1));
 
 				float gen1 = pv->gen1.value(k1);
 				float gen2 = pv->gen1.value(k2);
@@ -1484,11 +1526,12 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				const float reso1 = drumkv1_sigmoid_1(*elem->dcf1.reso
 					* env1 * (1.0f + *elem->lfo1.reso * lfo1));
 
-				gen1 = pv->dcf11.output(gen1, cutoff1, reso1);
-				gen2 = pv->dcf12.output(gen2, cutoff1, reso1);
 				if (int(*elem->dcf1.slope) > 0) { // 24db/octave
 					gen1 = pv->dcf13.output(gen1, cutoff1, reso1);
 					gen2 = pv->dcf14.output(gen2, cutoff1, reso1);
+				} else { // 12db/octave
+					gen1 = pv->dcf11.output(gen1, cutoff1, reso1);
+					gen2 = pv->dcf12.output(gen2, cutoff1, reso1);
 				}
 
 				// volumes
@@ -1517,6 +1560,10 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 			nblock -= ngen;
 
+			// voice ramps countdown
+
+			pv->dca1_pre.process(ngen);
+
 			// envelope countdowns
 
 			if (pv->dca1_env.running && pv->dca1_env.frames == 0)
@@ -1570,6 +1617,16 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				*out++ = drumkv1_sigmoid(*in++);
 		}
 	}
+
+	// post-processing
+
+	elem = m_elem_list.next();
+	while (elem) {
+		elem->wid1.process(nframes);
+		elem->pan1.process(nframes);
+		elem->vol1.process(nframes);
+		elem = elem->next();
+	}
 }
 
 
@@ -1738,7 +1795,7 @@ void drumkv1_element::setSampleFile ( const char *pszSampleFile )
 			m_pElem->gen1.sample0 = *(m_pElem->gen1.sample);
 		#endif
 			m_pElem->gen1_sample.open(pszSampleFile,
-				note_freq(m_pElem->gen1.sample0));
+				drumkv1_freq(m_pElem->gen1.sample0));
 		}
 	}
 }
