@@ -1,7 +1,7 @@
 // drumkv1_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -21,11 +21,10 @@
 
 #include "drumkv1_jack.h"
 
-#ifdef CONFIG_JACK_MIDI
 #include <jack/midiport.h>
-#endif
 
 #include <stdio.h>
+#include <string.h>
 
 
 #ifdef CONFIG_ALSA_MIDI
@@ -123,7 +122,28 @@ int drumkv1_jack_process ( jack_nframes_t nframes, void *arg )
 
 drumkv1_jack::drumkv1_jack (void) : drumkv1(2)
 {
-	open();
+	m_client = NULL;
+
+	m_audio_ins = NULL;
+	m_audio_outs = NULL;
+
+	m_ins = m_outs = NULL;
+
+	::memset(m_params, 0, NUM_PARAMS * sizeof(float));
+
+#ifdef CONFIG_JACK_MIDI
+	m_midi_in = NULL;
+#endif
+#ifdef CONFIG_ALSA_MIDI
+	m_alsa_seq     = NULL;
+//	m_alsa_client  = -1;
+	m_alsa_port    = -1;
+	m_alsa_decoder = NULL;
+	m_alsa_buffer  = NULL;
+	m_alsa_thread  = NULL;
+#endif
+
+//	open(DRUMKV1_TITLE);
 //	activate();
 }
 
@@ -131,7 +151,7 @@ drumkv1_jack::drumkv1_jack (void) : drumkv1(2)
 drumkv1_jack::~drumkv1_jack (void)
 {
 //	deactivate();
-	close();
+//	close();
 }
 
 
@@ -209,17 +229,17 @@ int drumkv1_jack::process ( jack_nframes_t nframes )
 }
 
 
-void drumkv1_jack::open (void)
+void drumkv1_jack::open ( const char *client_id )
 {
-	// open client
-	m_client = ::jack_client_open("drumkv1", JackNullOption, NULL);
-	if (m_client == NULL)
-		return;
-
 	// init param ports
 	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i)
 		drumkv1::setParamPort(drumkv1::ParamIndex(i), &m_params[i]);
 
+	// open client
+	m_client = ::jack_client_open(client_id, JackNullOption, NULL);
+	if (m_client == NULL)
+		return;
+
 	// set sample rate
 	drumkv1::setSampleRate(jack_get_sample_rate(m_client));
 //	drumkv1::reset();
@@ -259,7 +279,7 @@ void drumkv1_jack::open (void)
 	m_alsa_thread  = NULL;
 	// open alsa sequencer client...
 	if (snd_seq_open(&m_alsa_seq, "hw", SND_SEQ_OPEN_INPUT, 0) >= 0) {
-		snd_seq_set_client_name(m_alsa_seq, "drumkv1");
+		snd_seq_set_client_name(m_alsa_seq, client_id);
 	//	m_alsa_client = snd_seq_client_id(m_alsa_seq);
 		m_alsa_port = snd_seq_create_simple_port(m_alsa_seq, "in",
 			SND_SEQ_PORT_CAP_WRITE | SND_SEQ_PORT_CAP_SUBS_WRITE,
