@@ -1,7 +1,7 @@
 // drumkv1_lv2.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -21,15 +21,15 @@
 
 #include "drumkv1_lv2.h"
 
-#include "drumkv1widget_lv2.h"
+#include "drumkv1widget.h"
 
 #include "lv2/lv2plug.in/ns/ext/atom/atom.h"
 #include "lv2/lv2plug.in/ns/ext/midi/midi.h"
 #include "lv2/lv2plug.in/ns/ext/atom/util.h"
 
-#include "lv2/lv2plug.in/ns/ext/instance-access/instance-access.h"
 #include "lv2/lv2plug.in/ns/ext/state/state.h"
 
+#include <stdlib.h>
 #include <unistd.h>
 
 #include <sys/types.h>
@@ -37,6 +37,7 @@
 
 #include <QDomDocument>
 
+
 //-------------------------------------------------------------------------
 // drumkv1_lv2_map_path - abstract/absolute path functors.
 //
@@ -88,7 +89,7 @@ public:
 
 private:
 
-	LV2_State_Map_Path *m_map_path;		
+	LV2_State_Map_Path *m_map_path;
 };
 
 
@@ -236,6 +237,10 @@ void drumkv1_lv2::update_reset (void)
 }
 
 
+//-------------------------------------------------------------------------
+// drumkv1_lv2 - LV2 State interface.
+//
+
 static LV2_State_Status drumkv1_lv2_state_save ( LV2_Handle instance,
 	LV2_State_Store_Function store, LV2_State_Handle handle,
 	uint32_t flags, const LV2_Feature *const *features )
@@ -251,10 +256,12 @@ static LV2_State_Status drumkv1_lv2_state_save ( LV2_Handle instance,
 	uint32_t type = pPlugin->urid_map(LV2_ATOM__Chunk);
 	if (type == 0)
 		return LV2_STATE_ERR_BAD_TYPE;
-
+#if 0
 	if ((flags & (LV2_STATE_IS_POD | LV2_STATE_IS_PORTABLE)) == 0)
 		return LV2_STATE_ERR_BAD_FLAGS;
-
+#else
+	flags |= (LV2_STATE_IS_POD | LV2_STATE_IS_PORTABLE);
+#endif
 	drumkv1_lv2_map_path mapPath(features);
 
 	QDomDocument doc(DRUMKV1_TITLE);
@@ -325,6 +332,10 @@ static const LV2_State_Interface drumkv1_lv2_state_interface =
 };
 
 
+//-------------------------------------------------------------------------
+// drumkv1_lv2 - LV2 desc.
+//
+
 static LV2_Handle drumkv1_lv2_instantiate (
 	const LV2_Descriptor *, double sample_rate, const char *,
 	const LV2_Feature *const *host_features )
@@ -383,56 +394,6 @@ static const void *drumkv1_lv2_extension_data ( const char *uri )
 }
 
 
-static LV2UI_Handle drumkv1_lv2ui_instantiate (
-	const LV2UI_Descriptor *, const char *, const char *,
-	LV2UI_Write_Function write_function,
-	LV2UI_Controller controller, LV2UI_Widget *widget,
-	const LV2_Feature *const *features )
-{
-	drumkv1_lv2 *pSampl = NULL;
-
-	for (int i = 0; features && features[i]; ++i) {
-		if (::strcmp(features[i]->URI, LV2_INSTANCE_ACCESS_URI) == 0) {
-			pSampl = static_cast<drumkv1_lv2 *> (features[i]->data);
-			break;
-		}
-	}
-
-	if (pSampl == NULL)
-		return NULL;
-
-	drumkv1widget_lv2 *pWidget
-		= new drumkv1widget_lv2(pSampl, controller, write_function);
-	*widget = pWidget;
-
-	return pWidget;
-}
-
-
-static void drumkv1_lv2ui_cleanup ( LV2UI_Handle ui )
-{
-	drumkv1widget_lv2 *pWidget = static_cast<drumkv1widget_lv2 *> (ui);
-	if (pWidget)
-		delete pWidget;
-}
-
-
-static void drumkv1_lv2ui_port_event (
-	LV2UI_Handle ui, uint32_t port_index,
-	uint32_t buffer_size, uint32_t format, const void *buffer )
-{
-	drumkv1widget_lv2 *pWidget = static_cast<drumkv1widget_lv2 *> (ui);
-	if (pWidget)
-		pWidget->port_event(port_index, buffer_size, format, buffer);
-}
-
-
-static const void *drumkv1_lv2ui_extension_data ( const char * )
-{
-	return NULL;
-}
-
-
 static const LV2_Descriptor drumkv1_lv2_descriptor =
 {
 	DRUMKV1_LV2_URI,
@@ -445,15 +406,6 @@ static const LV2_Descriptor drumkv1_lv2_descriptor =
 	drumkv1_lv2_extension_data
 };
 
-static const LV2UI_Descriptor drumkv1_lv2ui_descriptor =
-{
-	DRUMKV1_LV2UI_URI,
-	drumkv1_lv2ui_instantiate,
-	drumkv1_lv2ui_cleanup,
-	drumkv1_lv2ui_port_event,
-	drumkv1_lv2ui_extension_data
-};
-
 
 LV2_SYMBOL_EXPORT const LV2_Descriptor *lv2_descriptor ( uint32_t index )
 {
@@ -461,10 +413,4 @@ LV2_SYMBOL_EXPORT const LV2_Descriptor *lv2_descriptor ( uint32_t index )
 }
 
 
-LV2_SYMBOL_EXPORT const LV2UI_Descriptor *lv2ui_descriptor ( uint32_t index )
-{
-	return (index == 0 ? &drumkv1_lv2ui_descriptor : NULL);
-}
-
-
 // end of drumkv1_lv2.cpp
