@@ -114,6 +114,25 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 {
 	Q_INIT_RESOURCE(drumkv1);
 
+#if QT_VERSION >= 0x050000
+	// HACK: Dark themes grayed/disabled color group fix...
+	QPalette pal;
+	if (pal.base().color().value() < 0x7f) {
+		const QColor& color = pal.window().color();
+		const int iGroups = int(QPalette::Active | QPalette::Inactive) + 1;
+		for (int i = 0; i < iGroups; ++i) {
+			const QPalette::ColorGroup group = QPalette::ColorGroup(i);
+			pal.setBrush(group, QPalette::Light,    color.lighter(150));
+			pal.setBrush(group, QPalette::Midlight, color.lighter(120));
+			pal.setBrush(group, QPalette::Dark,     color.darker(150));
+			pal.setBrush(group, QPalette::Mid,      color.darker(120));
+			pal.setBrush(group, QPalette::Shadow,   color.darker(200));
+		}
+		pal.setColor(QPalette::Disabled, QPalette::ButtonText, pal.mid().color());
+		QWidget::setPalette(pal);
+	}
+#endif
+
 	m_ui.setupUi(this);
 
 	// Init swapable params A/B to default.
@@ -485,6 +504,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_ui.StatusBar->showMessage(tr("Ready"), 5000);
 	m_ui.StatusBar->setModified(false);
+	m_ui.Preset->setDirtyPreset(false);
 }
 
 
@@ -537,10 +557,8 @@ void drumkv1widget::paramChanged ( float fValue )
 			.arg(m_ui.StackedWidget->currentWidget()->windowTitle())
 			.arg(pKnob->toolTip())
 			.arg(pKnob->valueText()), 5000);
-		m_ui.StatusBar->setModified(true);
+		updateDirtyPreset(true);
 	}
-
-	m_ui.Preset->dirtyPreset();
 }
 
 
@@ -561,7 +579,7 @@ void drumkv1widget::resetParams (void)
 	}
 
 	m_ui.StatusBar->showMessage(tr("Reset preset"), 5000);
-	m_ui.StatusBar->setModified(false);
+	updateDirtyPreset(false);
 }
 
 
@@ -613,11 +631,9 @@ void drumkv1widget::swapParams ( bool bOn )
 		}
 	}
 
-	m_ui.Preset->dirtyPreset();
-
 	const bool bSwapA = m_ui.SwapParamsAButton->isChecked();
 	m_ui.StatusBar->showMessage(tr("Swap %1").arg(bSwapA ? 'A' : 'B'), 5000);
-	m_ui.StatusBar->setModified(true);
+	updateDirtyPreset(true);
 }
 
  
@@ -692,7 +708,7 @@ void drumkv1widget::newPreset (void)
 	activateElement();
 
 	m_ui.StatusBar->showMessage(tr("New preset"), 5000);
-	m_ui.StatusBar->setModified(false);
+	updateDirtyPreset(false);
 }
 
 
@@ -773,7 +789,7 @@ void drumkv1widget::loadPreset ( const QString& sFilename )
 	m_ui.Preset->setPreset(sPreset);
 
 	m_ui.StatusBar->showMessage(tr("Load preset: %1").arg(sPreset), 5000);
-	m_ui.StatusBar->setModified(false);
+	updateDirtyPreset(false);
 
 	QDir::setCurrent(currentDir.absolutePath());
 
@@ -817,7 +833,7 @@ void drumkv1widget::savePreset ( const QString& sFilename )
 	}
 
 	m_ui.StatusBar->showMessage(tr("Save preset: %1").arg(sPreset), 5000);
-	m_ui.StatusBar->setModified(false);
+	updateDirtyPreset(false);
 }
 
 
@@ -929,9 +945,7 @@ void drumkv1widget::clearSample (void)
 	clearSampleFile();
 
 	m_ui.StatusBar->showMessage(tr("Clear sample"), 5000);
-	m_ui.StatusBar->setModified(true);
-
-	m_ui.Preset->dirtyPreset();
+	updateDirtyPreset(true);
 }
 
 
@@ -941,9 +955,7 @@ void drumkv1widget::loadSample ( const QString& sFilename )
 	loadSampleFile(sFilename);
 
 	m_ui.StatusBar->showMessage(tr("Load sample: %1").arg(sFilename), 5000);
-	m_ui.StatusBar->setModified(true);
-
-	m_ui.Preset->dirtyPreset();
+	updateDirtyPreset(true);
 }
 
 
@@ -1014,7 +1026,7 @@ void drumkv1widget::updateSample ( drumkv1_sample *pSample, bool bDirty )
 	m_ui.Gen1Sample->setSample(pSample);
 
 	if (pSample && bDirty)
-		m_ui.Preset->dirtyPreset();
+		updateDirtyPreset(true);
 }
 
 
@@ -1241,7 +1253,7 @@ void drumkv1widget::resetElement (void)
 	drumkv1 *pDrumk = instance();
 	if (pDrumk) {
 		pDrumk->removeElement(pDrumk->currentElement());
-		m_ui.Preset->dirtyPreset();
+		updateDirtyPreset(true);
 	}
 
 	refreshElements();
@@ -1361,4 +1373,12 @@ void drumkv1widget::helpAboutQt (void)
 }
 
 
+// Dirty flag (overridable virtual) methods.
+void drumkv1widget::updateDirtyPreset ( bool bDirtyPreset )
+{
+	m_ui.StatusBar->setModified(bDirtyPreset);
+	m_ui.Preset->setDirtyPreset(bDirtyPreset);
+}
+
+
 // end of drumkv1widget.cpp
